@@ -22,11 +22,14 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import collections
+import copy
 import functools
 
 from absl.flags import _argument_parser
 from absl.flags import _exceptions
 from absl.flags import _helpers
+import six
 
 
 @functools.total_ordering
@@ -125,6 +128,18 @@ class Flag(object):
       return id(self) < id(other)
     return NotImplemented
 
+  def __getstate__(self):
+    raise TypeError("can't pickle Flag objects")
+
+  def __copy__(self):
+    raise TypeError('%s does not support shallow copies. '
+                    'Use copy.deepcopy instead.' % type(self).__name__)
+
+  def __deepcopy__(self, memo):
+    result = object.__new__(type(self))
+    result.__dict__ = copy.deepcopy(self.__dict__, memo)
+    return result
+
   def _get_parsed_value_as_string(self, value):
     """Returns parsed flag value as string."""
     if value is None:
@@ -249,8 +264,9 @@ class Flag(object):
       default_serialized = self.default
     element.appendChild(_helpers.create_xml_dom_element(
         doc, 'default', default_serialized))
+    value_serialized = self._serialize_value_for_xml(self.value)
     element.appendChild(_helpers.create_xml_dom_element(
-        doc, 'current', self.value))
+        doc, 'current', value_serialized))
     element.appendChild(_helpers.create_xml_dom_element(
         doc, 'type', self.flag_type()))
     # Adds extra flag features this flag may have.
@@ -258,6 +274,10 @@ class Flag(object):
       element.appendChild(e)
     return element
 
+  def _serialize_value_for_xml(self, value):
+    """Returns the serialized value, for use in an XML help text."""
+    return value
+
   def _extra_xml_dom_elements(self, doc):
     """Returns extra info about this flag in XML.
 
@@ -318,7 +338,7 @@ class EnumClassFlag(Flag):
   def __init__(self, name, default, help, enum_class,  # pylint: disable=redefined-builtin
                short_name=None, **args):
     p = _argument_parser.EnumClassParser(enum_class)
-    g = _argument_parser.ArgumentSerializer()
+    g = _argument_parser.EnumClassSerializer()
     super(EnumClassFlag, self).__init__(
         p, g, name, default, help, short_name, **args)
     self.help = '<%s>: %s' % ('|'.join(enum_class.__members__), self.help)
@@ -340,8 +360,8 @@ class MultiFlag(Flag):
   See the __doc__ for Flag for most behavior of this class.  Only
   differences in behavior are described here:
 
-    * The default value may be either a single value or a list of values.
-      A single value is interpreted as the [value] singleton list.
+    * The default value may be either a single value or an iterable of values.
+      A single value is transformed into a single-item list of that value.
 
     * The value of the flag is always a list, even if the option was
       only supplied once, and even if the default value is a single
@@ -368,6 +388,10 @@ class MultiFlag(Flag):
     self.present += len(new_values)
 
   def _parse(self, arguments):
+    if (isinstance(arguments, collections.Iterable) and
+        not isinstance(arguments, six.string_types)):
+      arguments = list(arguments)
+
     if not isinstance(arguments, list):
       # Default value may be a list of values.  Most other arguments
       # will not be, so convert them into a single-item list to make
@@ -389,7 +413,7 @@ class MultiFlag(Flag):
     multi_value = self.value
 
     for self.value in multi_value:
-      if s: s += ' '
+      if s: s += '\n'
       s += Flag.serialize(self)
 
     self.value = multi_value
@@ -407,3 +431,37 @@ class MultiFlag(Flag):
         elements.append(_helpers.create_xml_dom_element(
             doc, 'enum_value', enum_value))
     return elements
+
+
+class MultiEnumClassFlag(MultiFlag):
+  """A multi_enum_class flag.
+
+  See the __doc__ for MultiFlag for most behaviors of this class.  In addition,
+  this class knows how to handle enum.Enum instances as values for this flag
+  type.
+  """
+
+  def __init__(self, name, default, help_string, enum_class, **args):
+    p = _argument_parser.EnumClassParser(enum_class)
+    g = _argument_parser.EnumClassListSerializer(list_sep=',')
+    super(MultiEnumClassFlag, self).__init__(
+        p, g, name, default, help_string, **args)
+    self.help = (
+        '<%s>: %s;\n    repeat this option to specify a list of values' %
+        ('|'.join(enum_class.__members__),
+         help_string or '(no help available)'))
+
+  def _extra_xml_dom_elements(self, doc):
+    elements = []
+    for enum_value in self.parser.enum_class.__members__.keys():
+      elements.append(_helpers.create_xml_dom_element(
+          doc, 'enum_value', enum_value))
+    return elements
+
+  def _serialize_value_for_xml(self, value):
+    """See base class."""
+    if value is not None:
+      value_serialized = self.serializer.serialize(value)
+    else:
+      value_serialized = ''
+    return value_serialized
