@@ -50,6 +50,12 @@ is printed to the log. To avoid this, use the level_debug() function:
   if logging.level_debug():
     logging.debug('Thing: %s', thing.ExpensiveOp())
 
+Per file level logging is supported by logging.vlog() and
+logging.vlog_is_on(). For example:
+
+    if logging.vlog_is_on(2):
+      logging.vlog(2, very_expensive_debug_message())
+
 Notes on Unicode:
 
 The log output is encoded as UTF-8.  Don't pass data in other encodings in
@@ -484,6 +490,31 @@ def vlog(level, msg, *args, **kwargs):
   log(level, msg, *args, **kwargs)
 
 
+def vlog_is_on(level):
+  """Checks if vlog is enabled for the given level in caller's source file.
+
+  Args:
+    level: int, the C++ verbose logging level at which to log the message,
+        e.g. 1, 2, 3, 4... While absl level constants are also supported,
+        callers should prefer level_debug|level_info|... calls for
+        checking those.
+
+  Returns:
+    True if logging is turned on for that level.
+  """
+
+  if level > converter.ABSL_DEBUG:
+    # Even though this function supports level that is greater than 1, users
+    # should use logging.vlog instead for such cases.
+    # Treat this as vlog, 1 is equivalent to DEBUG.
+    standard_level = converter.STANDARD_DEBUG - (level - 1)
+  else:
+    if level < converter.ABSL_FATAL:
+      level = converter.ABSL_FATAL
+    standard_level = converter.absl_to_standard(level)
+  return _absl_logger.isEnabledFor(standard_level)
+
+
 def flush():
   """Flushes all log files."""
   get_absl_handler().flush()
